#!/usr/bin/env python
"""
Web interface for [VWR circulating baths]
(https://us.vwr.com/store/catalog/product.jsp?catalog_number=89203-002).

Distributed under the GNU General Public License v2
Copyright (C) 2015 NuMat Technologies
"""
import getpass
import hashlib
import json
import os
import time
import traceback

from tornado.ioloop import PeriodicCallback
import tornado.web
import tornado.websocket

ROOT = os.path.normpath(os.path.dirname(__file__))
with open(os.path.join(ROOT, "password.txt")) as in_file:
    password = in_file.read().strip()


def set_password():
    """Called by `crystal-password`, this sets a new password."""
    digest = hashlib.sha512(getpass.getpass().encode("utf-8")).hexdigest()
    global password
    password = digest
    with open(os.path.join(ROOT, "password.txt"), "w") as out_file:
        out_file.write(digest)


def run_server(bath, port=10000, require_login=False):
    """Starts a web server on the specified port.

    Args:
        bath: An instance of `CirculatingBath` that is connected to the bath
            of interest.
        port: The port to serve the website. Default 10000.
        require_login: If True, serves a login page
    """
    class IndexHandler(tornado.web.RequestHandler):

        def get(self):
            if require_login and not self.get_secure_cookie("vwr"):
                self.redirect("/login")
            else:
                self.render("index.template", port=port)

    class LoginHandler(tornado.web.RequestHandler):

        def get(self):
            self.render("login.template")

        def post(self):
            submitted_pass = self.get_argument("password", "").encode("utf-8")
            if hashlib.sha512(submitted_pass).hexdigest() == password:
                self.set_secure_cookie("vwr", str(time.time()))
                self.redirect("/")
            else:
                time.sleep(1)
                self.redirect(u"/login?error")

    class WebSocket(tornado.websocket.WebSocketHandler):

        def open(self):
            self.p = PeriodicCallback(self._loop, 500)
            self.p.start()

        def _loop(self):
            state = {"setpoint": bath.get_setpoint(),
                     "actual": bath.get_internal_temperature(),
                     "units": bath.get_temperature_units()}
            res = json.dumps({"result": state, "error": 0, "data": True},
                             separators=(",", ":"))
            self.write_message(res)

        def on_message(self, message):
            """Evaluates the function pointed to by json-rpc."""
            json_rpc = json.loads(message)

            try:
                result = getattr(bath,
                                 json_rpc["method"])(**json_rpc["params"])
                error = None
            except:
                result = traceback.format_exc()
                error = 1

            self.write_message(json.dumps({"result": result,
                                           "error": error,
                                           "id": json_rpc["id"]},
                                          separators=(",", ":")))

    handlers = [(r"/", IndexHandler), (r"/login", LoginHandler),
                (r"/websocket", WebSocket),
                (r'/static/(.*)', tornado.web.StaticFileHandler,
                 {'path': ROOT})]
    application = tornado.web.Application(handlers, cookie_secret=password)
    application.listen(port)
    try:
        tornado.ioloop.IOLoop.instance().start()
    except:
        pass



#!/usr/bin/env python
"""Python driver for [VWR circulating baths]
(https://us.vwr.com/store/catalog/product.jsp?catalog_number=89203-002).

Distributed under the GNU General Public License v2
Copyright (C) 2015 NuMat Technologies
"""
import socket


class CirculatingBath(object):
    """Python driver for [VWR circulating baths](https://us.vwr.com/store/
    catalog/product.jsp?catalog_number=89203-002).

    This class communicates with the circulating bath over UDP sockets.
    """
    def __init__(self, address, password=100, timeout=2):
        """Opens ports to communicate with the circulating bath.

        Args:
            address: The IP address of the device
            password: The password next to "Unlock" in the user interface
                settings. Default 100.
            timeout: Max time to wait for response, in seconds. Default 2.
        """
        self.address = address
        self.password = password
        self.sender = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.listener = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.listener.bind(("", 1026))
        self.listener.settimeout(timeout)

    def turn_on(self):
        """Turns the circulating bath on.

        Returns:
            True if successful, else False.
        """
        self._send("SO1P{}".format(self.password))
        return (self._receive() == "!")

    def turn_off(self):
        """Turns the circulating bath off.

        Returns:
            True if successful, else False.
        """
        self._send("SO0P{}".format(self.password))
        return (self._receive() == "!")

    def get_setpoint(self):
        """Get the setpoint temperature."""
        self._send("RS")
        return float(self._receive())

    def get_temperature_units(self):
        """Gets the temperature units, e.g. C or F."""
        self._send("RU")
        return self._receive()

    def get_internal_temperature(self):
        """Get the temperature inside the bath."""
        self._send("RT")
        return float(self._receive())

    def get_external_temperature(self):
        """If connected, get the temperature of the external thermocouple."""
        self._send("RR")
        return self._receive()

    def set_setpoint(self, setpoint):
        """Sets setpoint temperature.

        Args:
            setpoint: Setpoint temperature, in machine units, as a float.
        Returns:
            True if successful, else False.
        """
        self._send("SS{:.2f}P{:d}".format(setpoint, self.password))
        return (self._receive() == "!")

    def close(self):
        """Closes the listening port."""
        self.listener.close()

    def _send(self, message):
        """Selds a message to the circulating bath."""
        self.sender.sendto((message + "\r").encode("utf-8"),
                           (self.address, 1024))

    def _receive(self):
        """Receives messages from the circulating bath."""
        message, _ = self.listener.recvfrom(512)
        return message.decode("utf-8").strip()



#!/usr/bin/env python
"""Python driver, command-line tool, and web server for [VWR circulating baths]
(https://us.vwr.com/store/catalog/product.jsp?catalog_number=89203-002).

Distributed under the GNU General Public License v2
Copyright (C) 2015 NuMat Technologies
"""
import argparse
import socket
import sys
import webbrowser
from blessings import Terminal
from vwr import server
from vwr.driver import CirculatingBath


def command_line():
    """Command-line interface to driver. Run with `vwr`."""
    parser = argparse.ArgumentParser(description="Control a VWR circulating "
                                     "bath from a web site.")
    parser.add_argument("address", help="The IP address of the bath.")
    parser.add_argument("--require-login", "-l", action="store_true",
                        help="Redirects users to a password login screen.")
    parser.add_argument("--set-password", "-p", action="store_true",
                        help="Start a dialog to set the password of the "
                        "website login page.")
    parser.add_argument("--set-temperature", "-t", default=None, type=float,
                        help="Sets the bath temperature.")
    parser.add_argument("--port", default=10000, type=int, help="The "
                        "port on which to run the web server. Default 10000.")
    parser.add_argument("--unlock-code", "-u", default=100, type=int,
                        help="The code opposite the 'Unlock' option on the "
                        "VWR circulating bath. Default 100.")
    args = parser.parse_args()

    bath = CirculatingBath(args.address, password=args.unlock_code)
    t = Terminal()

    if args.set_password:
        server.set_password()

    try:
        setpoint = bath.get_setpoint()
        actual = bath.get_internal_temperature()
        units = bath.get_temperature_units()
        print(t.bold_white("Current state:") + " {setpoint:.2f}°{units} "
              "setpoint, {actual:.2f}°{units} actual".format(**locals()))
    except socket.timeout:
        sys.stderr.write(t.bold_red("Could not connect to VWR circulating bath"
                         ". Is it running at {}?\r\n".format(args.address)))
        bath.close()
        sys.exit(0)

    if args.set_temperature:
        success = bath.set_setpoint(args.set_temperature)
        if success:
            print(t.bold_green("Successfully set temperature to {setpoint:.2f}"
                  "°{units}.".format(setpoint=args.set_temperature,
                                     units=units)))
        else:
            sys.stderr.write(t.bold_red("Failed to set temperature."))

    print(t.bold_white("Running server at http://localhost:{}/"
          .format(args.port)))
    webbrowser.open("http://localhost:{}/".format(args.port), new=2)
    try:
        server.run_server(bath, port=args.port,
                          require_login=args.require_login)
    finally:
        bath.close()


if __name__ == "__main__":
    command_line()



from setuptools import setup

setup(
    name="vwr",
    version="0.1.2",
    description="Python driver for VWR circulating baths.",
    url="http://github.com/numat/vwr/",
    author="Patrick Fuller",
    author_email="pat@numat-tech.com",
    packages=["vwr"],
    package_data={"vwr": ["vwr/client.js", "vwr/main.css",
                          "vwr/*.template", "vwr/password.txt"]},
    include_package_data=True,
    install_requires=["tornado", "blessings"],
    entry_points={
        "console_scripts": [("vwr = vwr:command_line")]
    },
    license="GPLv2",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Development Status :: 4 - Beta",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.4",
        "Topic :: Scientific/Engineering :: Human Machine Interfaces",
        "Topic :: Scientific/Engineering :: Electronic Design Automation (EDA)"
    ]
)
